"use strict";

const ee = require("./eventemitter");

const panelClass = "c-modal-dialog";
const eventNamespace = "tgam-modal-dialog";

module.exports = {

  init: function(openButton, panel) {
    this.panel = panel;
    this.openButton = openButton;
    this.closeButton = panel.getElementsByClassName("c-modal-dialog__close-button").item(0);

    // For accessibility reasons, we need to apply aria-hidden to all
    // elements that are obscured by the modal dialog when it's open
    this.elementsToHide = document.querySelectorAll("[data-c-modal-dialog-hides]");
    if (this.elementsToHide) {
      this.elementsToHide = [].slice.call(this.elementsToHide);
    } else {
      this.elementsToHide = [];
    }
    // Ensure that the "this" keyword always references the correct object
    // when these methods are used as event handlers (see below)
    this.boundOnKeyPress = this.onKeyPress.bind(this);
    this.boundOnFocusChange = this.onFocusChange.bind(this);

    if (this.panel && this.openButton && this.closeButton) {
      this.openButton.addEventListener("click", event => {
        if (event.target.nodeName.toLowerCase() === "a") {
          event.preventDefault();
        }
        this.open();
      });
      this.closeButton.addEventListener("click", () => {
        this.close();
      });
      this.panel.addEventListener("animationend", () => {
        this.onPanelAnimationEnd();
      });
    }
  },

  isOpen: function() {
    return this.openButton.getAttribute("aria-expanded") === "true";
  },

  open: function() {
    this.openButton.setAttribute("aria-expanded", true);
    this.panel.classList.remove(`${panelClass}--hiding`);
    this.panel.classList.add(`${panelClass}--revealing`);
    this.panel.removeAttribute("hidden");
    this.addKeyPressListener();
    this.activateKeyboardTrap();
    this.hideOutsideElements();
    ee.emitEvent(`${eventNamespace}:opening`);
  },

  close: function() {
    this.openButton.setAttribute("aria-expanded", false);
    this.panel.classList.remove(`${panelClass}--revealing`);
    this.panel.classList.add(`${panelClass}--hiding`);
    this.removeKeyPressListener();
    this.deactivateKeyboardTrap();
    this.revealOutsideElements();
    this.openButton.focus();
    ee.emitEvent(`${eventNamespace}:closing`);
  },

  onPanelAnimationEnd: function(e) {
    // Using classList.remove() with multiple arguments fails in IE11,
    // so we're removing classes separately
    this.panel.classList.remove(`${panelClass}--hiding`);
    this.panel.classList.remove(`${panelClass}--revealing`);
    if (this.openButton.getAttribute("aria-expanded") === "false") {
      this.panel.setAttribute("hidden", "hidden");
      ee.emitEvent(`${eventNamespace}:closed`);
    } else {
      this.closeButton.focus();
      ee.emitEvent(`${eventNamespace}:open`);
    }
  },

  addKeyPressListener: function() {
    document.addEventListener("keydown", this.boundOnKeyPress, false);
  },

  removeKeyPressListener: function() {
    document.removeEventListener("keydown", this.boundOnKeyPress);
  },

  onKeyPress: function(event) {
    if (this.isOpen() && event.key === "Escape") {
      this.close();
    }
  },

  activateKeyboardTrap: function() {
    document.addEventListener("focus", this.boundOnFocusChange, true);
  },

  deactivateKeyboardTrap: function() {
    document.removeEventListener("focus", this.boundOnFocusChange);
  },

  onFocusChange: function(event) {
    if (this.isOpen() && !this.isDescendant(this.panel, document.activeElement)) {
      // Set focus to the first focusable item inside the dialog
      this.closeButton.focus();
    }
  },

  hideOutsideElements: function() {
    this.elementsToHide.map(element => {
      element.setAttribute("aria-hidden", true);
    });
  },

  revealOutsideElements: function() {
    this.elementsToHide.map(element => {
      element.removeAttribute("aria-hidden");
    });
  },

  isDescendant: function(parent, child) {
    let node = child.parentNode;
    while (node != null) {
      if (node == parent) {
        return true;
      }
      node = node.parentNode;
    }
    return false;
  }

};
